<?php
/* $Id: navigateur.cls.php,v 1.10 2009/08/16 15:58:42 sarterm Exp $ */
include_once(dirname(__FILE__)."/config.inc.php");

/*
 * Stocke les informations sur un navigateur.
 * - agent : le user-agent original,
 * - agentMin : le user-argent en minuscules,
 * - navigateur : tableau contenant les informations sur le navigateur, dans un
 *   format similaire  celui de la fonction get_browser() de PHP. Les noms des
 *   champs commencent par une majuscule.
 */
class IBDNavigateur {
    public $agent;
    public $agentMin;
    public $navigateur = false;
    private $baseCache = false;
    private $baseBC = Array();
    private $baseFormat = Array();
    private $lite_baseBC = Array();
    private $lite_baseFormatLite = Array();
    private $init = false;
    private $lite_init = false;
    private $formattage = Array(
    	"\\" => "\\\\",
    	"." => "\\.",
    	"?" => ".",
    	"*" => ".*",
    	"^" => "\\^",
    	"[" => "\\[",
    	"]" => "\\]",
    	"|" => "\\|",
    	"(" => "\\(",
    	")" => "\\)",
    	"+" => "\\+",
    	"{" => "\\{",
    	"}" => "\\}",
    	'$' => '\\$');

    /*
     * Constructeur de la classe.
     * Le constructeur prend une chane user-agent en paramtre et effectue la
     * reconnaissance des informations dtailles.
     * 
     * @param agent La chane user-agent du navigateur. Si cette chane est
     *              vide, elle est rcupre dans la super globale $_SERVER.
     * @param fullBase Si  true, utilisation de la base php_browscap.ini.
     *                 Sinon, utilisation de la base liste_php_browscap.ini.
     * @param forceInit Si  true, la fonction init est appelle, mme si ce
     *                  n'est pas ncessaire.
     * @param clearCache Si  true, le cache des agents est effac.
     */
    function IBDNavigateur($agent = "", $fullBase = true, $forceInit = false, $clearCache = false) {
        global $cfgIBDIN;
        $pref = "";
        $now = explode(' ',microtime());
        $start = $now[0] + $now[1];
        $this->agent = $agent;
        $this->agentMin = strtolower($agent);
        
        if ($cfgIBDIN["useCacheAgents"]) {
            $this->navigateur = $this->extraireCache($agent);
            if ($this->navigateur !== false) {
                $now = explode(' ',microtime());
                $end = $now[0] + $now[1];
                return $end - $start;
            }
        }
        
        if (!$fullBase) {
            $pref = "lite_";
        }
        if ((!$this->{$pref."init"} || $forceInit)) {
            $this->init($pref);
        }
        
        if ($clearCache) {
            unlink(dirname(__FILE__)."/baseCache.dat");
        }
          
        if ($agent == "") {
            $agent = $_SERVER["HTTP_USER_AGENT"];   
            if ($agent == "") {
                return;
            }
        }  
        $this->navigateur = false;
        
        foreach($this->{$pref."baseFormat"} as $ereg => $traduction)
        {
            $valeur = $this->{$pref."baseBC"}[$traduction];
            if (!array_key_exists("Parent", $valeur)) {
                continue;
            }
            if (preg_match("".$ereg."", $this->agentMin))
            {
                $this->navigateur = Array("Agent" => $agent, "browser_name_regex" => $ereg, "browser_name_patern" => $traduction) + $valeur;
                while (array_key_exists("Parent", $valeur) && array_key_exists($valeur["Parent"], $this->{$pref."baseBC"})) {
                    $parent = $valeur["Parent"];
                    $valeur = $this->{$pref."baseBC"}[$parent];
                    $this->navigateur += $valeur;
                }
                break;
            }
        }
        
        if ($cfgIBDIN["useCacheAgents"] &&
            $this->navigateur !== false) {
            $this->ajouterCache($agent, $this->navigateur);
        }
        
        $now = explode(' ',microtime());
        $end = $now[0] + $now[1];
        return $end - $start;
    }
    
    /*
     * Initialise baseBC et baseFormat  partir de fichier php_browscap.ini ou
     *  partir du cache.
     */
    private function init($pref = "") {
        global $cfgIBDIN;
        if ($cfgIBDIN["useCache"] && file_exists(dirname(__FILE__)."/".$pref."baseBC.dat")) {
            $this->{$pref."baseBC"} = unserialize(file_get_contents(dirname(__FILE__)."/".$pref."baseBC.dat"));
        }
        else {
            $this->{$pref."baseBC"} = parse_ini_file(dirname(__FILE__)."/".$pref."php_browscap.ini", true);
            if ($cfgIBDIN["useCache"]) {
                file_put_contents(dirname(__FILE__)."/".$pref."baseBC.dat", serialize($this->{$pref."baseBC"}));
            }
        }
        if ($cfgIBDIN["useCache"] && file_exists(dirname(__FILE__)."/".$pref."baseFormat.dat")) {
            $this->{$pref."baseFormat"} = unserialize(file_get_contents(dirname(__FILE__)."/".$pref."baseFormat.dat"));
        }
        else {
            foreach ($this->{$pref."baseBC"} as $cle => $valeur) {
                $this->{$pref."baseFormat"}[$this->formateEreg($cle)] = $cle;
            }
            if ($cfgIBDIN["useCache"]) {
                file_put_contents(dirname(__FILE__)."/".$pref."baseFormat.dat", serialize($this->{$pref."baseFormat"}));
            }
        }
        $this->{$pref."init"} = true;
    }
        
    /*
     * Converti les expressions rgulires du format php_browscap.ini en un
     * format exploitable par preg_match.
	 *
	 * @param ereg L'expression rgulire  convertir.
	 * @return L'expression rgulire convertie.
     */
    private function formateEreg($ereg) {
        $ereg = strtolower($ereg);
        reset($this->formattage);
        foreach ($this->formattage as $cle => $valeur) {
            $ereg = str_replace($cle, $valeur, $ereg);
        }
        return "^".$ereg."$";
    }
    
    /*
     * Cherche un agent dans le cache des agents dj traits.
     * 
     * @param agent La chane user-agent  chercher.
	 * @return Le tableau dcrivant le navigateur si l'agent a t trouv dans
	 *         le cache. false sinon.
     */
    private function extraireCache($agent) {
        if ($this->baseCache === false) {
            if (file_exists(dirname(__FILE__)."/baseCache.dat")) {
                $this->baseCache = unserialize(file_get_contents(dirname(__FILE__)."/baseCache.dat"));
            }
        }
        if (isset($this->baseCache[$agent])) {
            return $this->baseCache[$agent];
        }
        return false;
    }
    
    /*
     * Ajoute un agent dans le cache.
     * 
     * @param agent La chane user-agent  ajouter.
	 * @param navigateur Le tableau dcrivant le navigateur correspondant.
     */
    private function ajouterCache($agent, $navigateur) {
        if ($this->baseCache === false) {
            $this->baseCache = Array();
            if (file_exists(dirname(__FILE__)."/baseCache.dat")) {
                $this->baseCache = unserialize(file_get_contents(dirname(__FILE__)."/baseCache.dat"));
            }
        }
        if (!isset($this->baseCache[$agent])) {
            $this->baseCache[$agent] = $navigateur;
            file_put_contents(dirname(__FILE__)."/baseCache.dat", serialize($this->baseCache));
        }
    }
    
    /*
     * Renvoie un tableau dcrivant le navigateur dans un format identique 
     * celui renvoy par la fonction get_browser de PHP.
     *
     * @return Un tableau contenant les mmes champs qu'un tableau renvoy par
     *         get_browser().
     */
    public function get_browser() {
        if ($this->navigateur === false) {
            return Array();
        }
        foreach ($this->navigateur as $cle => $valeur) {
            if ($cle != "Agent") {
                $ret[strtolower($cle)] = $valeur;
            }
        }
        return $ret;
    }
}
?>